
// Microsoft Windows/NT openGL include files
#include "glos.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glaux.h>

// local object include files
#include "faccess.h"
#include "globj.h"
#include "glface.h"

#define DEBUG 0

static FaceList *CurObj;

void SetObj( FaceList *obj )
{
  CurObj = obj;
}


//
// Get the view values (e.g., the min and max values for the X, Y and Z
// axies.
//
typedef struct { float min;
                 float max; 
               } AxisMinMax;
static AxisMinMax Axis;

void init_minmax( GL_object &GL_Obj )
{
  Axis.min = GL_Obj.PointRange.GetMin();
  Axis.max = GL_Obj.PointRange.GetMax();
  Axis.min = (float)((int)Axis.min + 1);
  Axis.max = (float)((int)Axis.max + 1);
}


void print_minmax(void)
{
  printf("axis range = %2.2f to %2.2f\n", Axis.min, Axis.max );
}


//
// Feed the point list to openGL as a polygon
//
void disp_gl_face(FaceList::face *face_ptr )
{
  int NumPts = face_ptr->point_cnt;
  int i;

  glBegin( GL_POLYGON );
  for (i = 0; i < NumPts; i++) {
    glVertex3fv(face_ptr->f[i].v);
  }
  glEnd();
} // disp_gl_face



static int SpinAngle = 0;       // spin angle, in degrees
//
// The glRotatef function takes four arguments, the angle of rotation,
// and a 3-D point.  The axis of rotation defined by the line from (0,0,0)
// to the 3-D point.  So to rotate on the X-axis, the point would be
// (1, 0, 0), to rotate on the Y-axis (0, 1, 0) and to rotate on the
// Z-axis (0, 0, 1).
//
const int On  = 1;
const int Off = 0;
// Rotate first on the X-axis
static int AxisPoint[ 3 ] = {On, Off, Off};
static int AxisIx = 0;



//
// Display the wire frame described by the face list pointed to by
// CurObj.
//
void CALLBACK display(void)
{

  glClear(GL_COLOR_BUFFER_BIT);
  
  if (CurObj != NULL) {
    glPushMatrix();

    glRotatef((float)SpinAngle, (float)AxisPoint[0],
	                        (float)AxisPoint[1],
	                        (float)AxisPoint[2]);

    FaceList::ListHandle FaceList = CurObj->GetListStart();
    while (FaceList != NULL) {
      disp_gl_face( CurObj->GetFace(FaceList) );
      FaceList = CurObj->GetNextFace( FaceList );
    } // while

    glPopMatrix();
    glFlush();
    auxSwapBuffers();
  } // CurObj != NULL
}



void CALLBACK spinDisplay(void)
{
  SpinAngle = SpinAngle + 2;
  if (SpinAngle > 360) {
    AxisPoint[ AxisIx ] = Off;
    AxisIx = (AxisIx + 1) % 3;
    AxisPoint[ AxisIx ] = On;
    SpinAngle = SpinAngle - 360;
  }
  display();
}




void CALLBACK myReshape(GLsizei w, GLsizei h)
{
  glViewport(0, 0, w, h);

  glMatrixMode (GL_PROJECTION);
  glLoadIdentity();

  if (w <= h) {
    float AdjY = Axis.max * (GLfloat)h / (GLfloat)w;
    glOrtho(-Axis.max, Axis.max,  // x-axis
	    -AdjY, AdjY,          // y-axis
	    -Axis.max, Axis.max); // z-axis
  }
  else {
    float AdjX = Axis.max * (GLfloat)h / (GLfloat)w;
    glOrtho(-AdjX, AdjX,          // x-axis
	    -Axis.max, Axis.max,  // y-axis
	    -Axis.max, Axis.max); // z-axis
  }

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}


void myinit (void) 
{
  //  clear background to black 
  glClearColor (0.0, 0.0, 0.0, 0.0);   
  // draw all lines in white, on the black background
  glColor3f(1.0, 1.0, 1.0);
  // Set polygon mode to line
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  // "cull" invisible polygons
  glCullFace(GL_BACK);
  // turn on culling
  glEnable(GL_CULL_FACE);
}  // myinit



/* Read a DXF file and display the object using openGL
 */
void display_gl_obj(char *ProgName, char *FileName)
{
  FileAccess FileObj( FileName );

  if (FileObj.FileOK()) {
    GL_object GL_Obj( &FileObj );
    FaceList *CurObj = GL_Obj.GL_GetFaceList();  // Get the DXF object
    SetObj( CurObj );
    
    init_minmax( GL_Obj );  // save the min and the max axis values
#if DEBUG
    print_minmax();
    CurObj->print();
#endif

    // GL display stuff
    auxInitDisplayMode (AUX_DOUBLE | AUX_RGBA );
    auxInitPosition (0, 0, 500, 500);
    auxInitWindow (ProgName);
    myinit();

    auxReshapeFunc(myReshape);
    auxIdleFunc(spinDisplay);
    auxMainLoop(display);

    delete CurObj;
  }
} // display_gl_obj



void main(int argc, char *argv[])
{
  char *FileName;

  if (argc == 2) {
    FileName = argv[ 1 ];
    display_gl_obj(argv[0], FileName);
  }
  else {
    printf("usage: %s <filename>\n", argv[0] );
  }
}
